#ifndef __LOG_H__
#define __LOG_H__

#include <android/log.h>

#include <errno.h>
#include <string.h>

#define LOG_TAG "virtual_input_device"

#define LOG_ERROR_OPEN  "error open"
#define LOG_ERROR_CLOSE "error close"
#define LOG_ERROR_READ  "error read"
#define LOG_ERROR_WRITE "error write"
#define LOG_ERROR_IOCTL "error ioctl"
#define LOG_ERROR_INPUT "error input"

#define LOG(prio, tag, log, ...) __android_log_print(prio, tag, log, ##__VA_ARGS__)

#define LOG_VERBOSE(log, ...) LOG(ANDROID_LOG_VERBOSE, LOG_TAG, log, ##__VA_ARGS__)
#define LOG_DEBUG(  log, ...) LOG(ANDROID_LOG_DEBUG,   LOG_TAG, log, ##__VA_ARGS__)
#define LOG_INFO(   log, ...) LOG(ANDROID_LOG_INFO,    LOG_TAG, log, ##__VA_ARGS__)
#define LOG_WARN(   log, ...) LOG(ANDROID_LOG_WARN,    LOG_TAG, log, ##__VA_ARGS__)
#define LOG_ERROR(  log, ...) LOG(ANDROID_LOG_ERROR,   LOG_TAG, log, ##__VA_ARGS__)
#define LOG_FATAL(  log, ...) LOG(ANDROID_LOG_FATAL,   LOG_TAG, log, ##__VA_ARGS__)

#define LOG_ERRNO(  log, ...) LOG(ANDROID_LOG_ERROR,   LOG_TAG, log": %s", ##__VA_ARGS__, \
                                                                           strerror(errno))

#define ASSIGN(var, ret, log) do { if ((var = ret) < 0) { LOG_ERROR(log); return var; } } while (0)
#define CHECK(ret, log) do { int var; ASSIGN(var, ret, log); } while (0)

#endif /* __LOG_H__ */
