#ifndef __SIGNAL_H__
#define __SIGNAL_H__

#include "log.h"

#include <sys/eventfd.h>

#include <errno.h>
#include <stddef.h>
#include <stdint.h>
#include <unistd.h>

static int signal_prime() {
    int efd;

    efd = eventfd(0, 0);
    if (efd < 0) {
        LOG_ERRNO("Cannot open event signal");
        return -errno;
    } else {
        return efd;
    }
}

static int signal_defuse(const int efd) {
    int ret;

    ret = close(efd);
    if (ret < 0) {
        LOG_ERRNO("Cannot close event signal");
        return -errno;
    } else {
        return ret;
    }
}

static int signal_fire(const int efd) {
    int ret;
    size_t num;
    uint64_t val;

    val = 1;
    num = sizeof(val);
    ret = write(efd, &val, num);
    if (ret < 0) {
        LOG_ERRNO("Cannot write to event signal");
        return -errno;
    } else if (ret != num) {
        LOG_ERROR("Wrong size written to event signal: %d != %zu", ret, num);
        errno = EFAULT;
        return -errno;
    } else {
        return ret;
    }
}

#endif /* __SIGNAL_H__ */
