package com.genymobile.scrcpy;

import android.os.Build;

public class VirtualPointer {

    private enum State { LOADED, INITIALIZED, KILLED, FAILED }

    private static State state;

    private static native boolean nativeInit();
    private static native boolean nativeKill();
    private static native boolean nativeMove(int dx, int dy, boolean calibrating);

    private VirtualPointer() {
        // Static class
    }

    static {
        for (String abi : Build.SUPPORTED_ABIS) {
            try {
                NativeUtils.loadLibraryFromJar("/lib/" + abi + "/libpointer.so");
                Ln.i("Loaded native pointer library after extracting from archive for ABI " + abi);
                state = State.LOADED;
                break;
            } catch (Exception ignored) {
            }
        }
        if (state != State.LOADED) {
            Ln.e("Unable to load native pointer library");
        }
    }

    synchronized public static void init() {
        if (state == State.LOADED) {
            if (nativeInit()) {
                Ln.i("Successfully initialized native pointer library");
                state = State.INITIALIZED;
            } else {
                Ln.w("Failed to initialize native pointer library");
                state = State.FAILED;
            }
        } else {
            Ln.e("Native pointer library is in an invalid state (" + state + ") for initializing");
        }
    }

    synchronized public static void kill() {
        if (state == State.INITIALIZED) {
            if (nativeKill()) {
                Ln.i("Successfully killed native pointer library");
                state = State.KILLED;
            } else {
                Ln.w("Failed to kill native pointer library");
                state = State.FAILED;
            }
        } else {
            Ln.e("Native pointer library is in an invalid state (" + state + ") for killing");
        }
    }

    synchronized private static void move(int dx, int dy, boolean calibrating) {
        if (state == State.INITIALIZED) {
            if (!nativeMove(dx, dy, calibrating)) {
                Ln.w("Failed to move with native pointer library");
            }
        } else {
            Ln.e("Native pointer library is in an invalid state (" + state + ") for moving");
        }
    }

    public static void move(int dx, int dy) {
        move(dx, dy, false);
    }

    public static void calibrate(int dx, int dy) {
        move(dx, dy, true);
    }
}
